from typing import List

import numpy as np

from gui.layer import GridLayer
from .OpenGLLayer import OpenGLLayer
from .OpenGLLayerGroup import OpenGLLayerGroup
from .. import OpenGLGUIFacade, ZBuffer


class OpenGLGridLayer(OpenGLLayer, GridLayer):

    def __init__(self, gui: OpenGLGUIFacade, layerGroup: OpenGLLayerGroup):
        super(OpenGLGridLayer, self).__init__(gui, layerGroup, dynamic=False)
        self.__width = 0  # type: int
        self.__height = 0  # type: int
        self.__depths = []  # type: List[int]

    def updateDepths(self, minDepth: int, maxDepth: int, index: int, maxIndex: int) -> int:
        faceIndex = 0
        self.__depths = []
        for y in range(self.__height):
            rowDepth = maxDepth - index - y * maxIndex
            self.setFacesDepth(rowDepth, range(faceIndex, faceIndex+self.__width))
            self.__depths.append(rowDepth)
            faceIndex += self.__width
        return maxDepth - index - self.__height * maxIndex

    def hasDepth(self, depth: int) -> bool:
        return depth in self.__depths

    def getPixelCell(self, x: int, y: int) -> (int, int):
        depth = int(round(self._gui.getPixelDepth(x, y)))
        viewX, viewY = self._layerGroup.getTranslation()
        cellX = (x + viewX) // self.tileWidth
        for cellY, rowDepth in enumerate(self.__depths):
            if rowDepth == depth:
                return cellX, cellY
        return -1, -1

    def setTiles(self, grid: np.ndarray):
        assert grid.ndim == 3
        self.__width = gridWidth = grid.shape[0]
        self.__height = gridHeight = grid.shape[1]
        assert grid.shape[2] == 2

        self.createFaces(gridWidth * gridHeight, 0)
        faceIndex = 0
        for y in range(gridHeight):
            for x in range(gridWidth):
                self.setFaceTileLocation(faceIndex, x, y)
                self.setFaceTileTexture(faceIndex, grid[x, y, 0], grid[x, y, 1])
                faceIndex += 1

    def setTile(self, x: int, y: int, tileX: int, tileY: int, width: int = 1, height: int = 1):
        faceIndex = x + y * self.__width
        self.setFaceTileLocation(faceIndex, x, y, width, height)
        self.setFaceTileTexture(faceIndex, tileX, tileY, width, height)
        if height == 1:
            self.setFaceDepth(faceIndex, self.__depths[0])
        else:
            self.setFaceDepth(faceIndex, self.__depths[y])

