from typing import List

from ..layer import OpenGLLayer


class FrameBox:
    __slots__ = ['layer', 'facesIndices', 'x1', 'y1', 'x2', 'y2', 'tileWidth', 'tileHeight']

    def __init__(self, layer: OpenGLLayer, facesIndices: List[int], x1: float, y1: float, x2: float, y2: float,
                 tileWidth: float, tileHeight: float):
        self.layer = layer
        self.facesIndices = facesIndices
        self.x1 = x1
        self.y1 = y1
        self.x2 = x2
        self.y2 = y2
        self.tileWidth = tileWidth
        self.tileHeight = tileHeight

    def show(self):
        # Local variables (for readability)
        layer = self.layer
        facesIndices = self.facesIndices
        x1 = self.x1
        y1 = self.y1
        x2 = self.x2
        y2 = self.y2
        tileWidth = self.tileWidth
        tileHeight = self.tileHeight

        # Top left
        layer.setFacePixelLocation(facesIndices[0], x1, y1, x1 + tileWidth, y1 + tileHeight)
        layer.setFacePixelTexture(facesIndices[0], 0, 0, tileWidth, tileHeight)

        # Top
        layer.setFacePixelLocation(facesIndices[1], x1 + tileWidth, y1, x2 - tileWidth, y1 + tileHeight)
        layer.setFacePixelTexture(facesIndices[1], tileWidth, 0, 2 * tileWidth, tileHeight)

        # Top right
        layer.setFacePixelLocation(facesIndices[2], x2 - tileWidth, y1, x2, y1 + tileHeight)
        layer.setFacePixelTexture(facesIndices[2], 2 * tileWidth, 0, 3 * tileWidth, tileHeight)

        # Right
        layer.setFacePixelLocation(facesIndices[3], x2 - tileWidth, y1 + tileHeight, x2, y2 - tileHeight)
        layer.setFacePixelTexture(facesIndices[3], 2 * tileWidth, tileHeight, 3 * tileWidth, 2 * tileHeight)

        # Bottom right
        layer.setFacePixelLocation(facesIndices[4], x2 - tileWidth, y2 - tileHeight, x2, y2)
        layer.setFacePixelTexture(facesIndices[4], 2 * tileWidth, 2 * tileHeight, 3 * tileWidth, 3 * tileHeight)

        # Bottom
        layer.setFacePixelLocation(facesIndices[5], x1 + tileWidth, y2 - tileHeight, x2 - tileWidth, y2)
        layer.setFacePixelTexture(facesIndices[5], tileWidth, 2 * tileHeight, 2 * tileWidth, 3 * tileHeight)

        # Bottom left
        layer.setFacePixelLocation(facesIndices[6], x1, y2 - tileHeight, x1 + tileWidth, y2)
        layer.setFacePixelTexture(facesIndices[6], 0, 2 * tileHeight, tileWidth, 3 * tileHeight)

        # Left
        layer.setFacePixelLocation(facesIndices[7], x1, y1 + tileHeight, x1 + tileWidth, y2 - tileHeight)
        layer.setFacePixelTexture(facesIndices[7], 0, tileHeight, tileWidth, 2 * tileHeight)

        # Center
        layer.setFacePixelLocation(facesIndices[8], x1 + tileWidth, y1 + tileHeight, x2 - tileWidth, y2 - tileHeight)
        layer.setFacePixelTexture(facesIndices[8], tileWidth, tileHeight, 2 * tileWidth, 2 * tileHeight)

    def hide(self):
        self.layer.hideFaces(self.facesIndices)

    def __str__(self):
        return "{} ({},{},{},{}) ({},{})".format(
            self.facesIndices,
            self.x1,
            self.y1,
            self.x2,
            self.y2,
            self.tileWidth,
            self.tileHeight
        )