import pygame

from . import CharacterRenderer
from gui.text import TextStyle


class UnderlineCharacterRenderer(CharacterRenderer):

    def __init__(self, style: TextStyle, characterRenderer: CharacterRenderer):
        self.__characterRenderer = characterRenderer
        self.__color = style.color
        self.__underlineShift = style.underlineShift
        self.__underlineThickness = style.underlineThickness

    @property
    def tileDefaultWidth(self) -> int:
        return self.__characterRenderer.tileDefaultWidth

    @property
    def tileDefaultHeight(self) -> int:
        return self.__characterRenderer.tileDefaultHeight

    def getCharacterAdvance(self, character: str) -> int:
        return self.__characterRenderer.getCharacterAdvance(character)

    def render(self, text: str) -> pygame.Surface:
        surface = self.__characterRenderer.render(text)

        width = surface.get_width()
        start = (0, self.__underlineShift)
        end = (width - 1, self.__underlineShift)
        pygame.draw.line(surface, self.__color, start, end, self.__underlineThickness)

        return surface
