from __future__ import annotations

from logic import Logic
from state import State
from logic.command.Command import Command, REGION_PRIORITY
from state.Region import REGION_MAX_WIDTH


class FillWaterValueCommand(Command):

    def __init__(self, state: State, regionName: str, x: int, y: int, background: int, foreground: int):
        self.__state = state
        self.__regionName = regionName
        self.__x = x
        self.__y = y
        self.__background = background
        self.__foreground = foreground

    def priority(self) -> int:
        return REGION_PRIORITY + self.__x + self.__y * REGION_MAX_WIDTH

    def execute(self, logic: Logic):
        region = self.__state.getRegion(self.__regionName)
        if not (0 <= self.__x < region.width and 0 <= self.__y < region.height):
            return
        currentValue = region.getWaterValue(self.__x, self.__y)
        if currentValue == self.__foreground or currentValue != self.__background:
            return

        region.setWaterValue(self.__x, self.__y, self.__foreground)
        self.__state.notifyRegionCellChanged(self.__regionName, self.__x, self.__y)

        logic.addCommand(FillWaterValueCommand(self.__state, self.__regionName, self.__x + 1, self.__y, self.__background, self.__foreground))
        logic.addCommand(FillWaterValueCommand(self.__state, self.__regionName, self.__x - 1, self.__y, self.__background, self.__foreground))
        logic.addCommand(FillWaterValueCommand(self.__state, self.__regionName, self.__x, self.__y + 1, self.__background, self.__foreground))
        logic.addCommand(FillWaterValueCommand(self.__state, self.__regionName, self.__x, self.__y - 1, self.__background, self.__foreground))
