from typing import Union

import numpy as np

from gui import GUIFacade
from gui.layer import GridLayer, LayerGroup
from state import Region
from state.Region import TREE_DEAD
from .Renderer import Renderer


class TreeRenderer(Renderer):

    def __init__(self, gui: GUIFacade, layerGroup: LayerGroup, layerIndex: int, tileSize: int):
        super(TreeRenderer, self).__init__(gui)
        self.__layerGroup = layerGroup
        self.__layerIndex = layerIndex
        self.__tileSize = tileSize  # type: int
        self.__layer = None  # type: Union[None, GridLayer]
        self._animationStep = 0

    def updateLayer(self, region: Union[None, Region]):
        # Case where we remove the region
        if region is None:
            if self.__layer is not None:
                self.__layerGroup.removeLayer(self.__layer)
                self.__layer = None
            return

        # Create the layer
        self.__layer = self.__layerGroup.createGridLayer(self.__layerIndex)
        self.__layer.setTileset("assets/tile/tree.png")
        self.__layer.setTileSize(self.__tileSize, self.__tileSize)
        self.__layer.setTiles(np.zeros([region.width, region.height, 2], dtype=np.int32))

    def updateTiles(self, region: Region):
        # Update tiles
        for y in range(region.height):
            for x in range(region.width):
                self.updateTile(region, x, y)

    def updateTile(self, region: Region, x: int, y: int):
        value = region.getTreeValue(x, y)
        if value == 0:
            self.__layer.setTile(x, y, 7, 7)
            return

        if value >= len(self.treeTiles):
            return
        width = region.width
        height = region.height
        if value <= TREE_DEAD:
            tileCoords = self.treeTiles[value]
            if ((x + y) % 2) == 0:
                if (x + 1) < width and region.getTreeValue(x + 1, y) == value:
                    self.__layer.setTile(x, y, 2 * tileCoords[0],  tileCoords[1] - 3, 1, 2)
                else:
                    self.__layer.setTile(x, y, tileCoords[0], tileCoords[1], 1, 1)
            else:
                if x > 0 and region.getTreeValue(x - 1, y) == value:
                    self.__layer.setTile(x, y,  2 * tileCoords[0] + 1,  tileCoords[1] - 3, 1, 2)
                else:
                    self.__layer.setTile(x, y, tileCoords[0], tileCoords[1], 1, 1)
        else:
            tileCoords = self.treeTiles[value]
            self.__layer.setTile(x, y,  tileCoords[0],  tileCoords[1])

    treeTiles = np.array([
        [7, 7],
        [0, 4], [1, 4], [2, 4], [3, 4],
        [4, 4], [5, 4],
        [0, 5], [1, 5], [2, 5], [3, 5],
        [4, 5], [5, 5], [6, 5], [7, 5],
        [0, 6], [1, 6],
        [2, 6], [3, 6],
    ], dtype=np.int32)